/*
 * Agilent6613C device support
 */

#include <epicsStdio.h>
#include <devCommonGpib.h>

/******************************************************************************
 *
 * The following define statements are used to declare the names to be used
 * for the dset tables.   
 *
 * A DSET_AI entry must be declared here and referenced in an application
 * database description file even if the device provides no AI records.
 *
 ******************************************************************************/
#define DSET_AI     devAiAgilent6613C
#define DSET_AO     devAoAgilent6613C
#define DSET_BI     devBiAgilent6613C
#define DSET_BO     devBoAgilent6613C
#define DSET_SI     devSiAgilent6613C

#include <devGpib.h> /* must be included after DSET defines */

#define TIMEOUT     5.0    /* I/O must complete within this time */
#define TIMEWINDOW  2.0    /* Wait this long after device timeout */

/******************************************************************************
 * Strings used by the init routines to fill in the znam,onam,...
 * fields in BI and BO record types.
 ******************************************************************************/
static char  *offOnList[] = { "Off","On" };
static struct devGpibNames   outputOffOn = { 2,offOnList,0,1 };

static char  *currOffOnList[] = { "Off","On" };
static struct devGpibNames   currProtStat = { 2,currOffOnList,0,1 };

/******************************************************************************
 * String arrays for EFAST operations. The last entry must be 0.
 *
 * On input operations,only as many bytes as are found in the string array
 * elements are compared.  Additional bytes are ignored.
 * The first matching string  will be used as a match.
 *
 * For the input operations,the strings are compared literally!  This
 * can cause problems if the instrument is returning things like \r and \n
 * characters.  When defining input strings so you include them as well.
 ******************************************************************************/

static char *offOn[] = {"OFF","ON",0};

/******************************************************************************
 * Array of structures that define all GPIB messages
 * supported for this type of instrument.
 ******************************************************************************/

static struct gpibCmd gpibCmds[] = {
   
  /* Param 0 get ID */
  {&DSET_SI,GPIBREAD,IB_Q_LOW,"*IDN?","%39[^\n]",0,80,0,0,0,NULL,NULL,0},
  /* Param 1 set output voltage : AO*/
  {&DSET_AO,GPIBWRITE,IB_Q_HIGH,NULL,"VOLTAGE %.3f",0,80,0,0,0,NULL,NULL,0},
  /* Param 2 read output voltage setting : AI*/
  {&DSET_AI,GPIBREAD,IB_Q_LOW,"VOLTAGE?","%lf",0,80,0,0,0,NULL,NULL,0},
  /* Param 3 read measured output voltage : AI*/
  {&DSET_AI,GPIBREAD,IB_Q_LOW,"MEAS:VOLTAGE?","%lf",0,80,0,0,0,NULL,NULL,0},
  /* Param 4 set output current : AO*/
  {&DSET_AO,GPIBWRITE,IB_Q_HIGH,NULL,"CURR %.5f",0,80,0,0,0,NULL,NULL,0},
  /* Param 5 read output current setting : AI*/
  {&DSET_AI,GPIBREAD,IB_Q_LOW,"CURR?","%lf",0,80,0,0,0,NULL,NULL,0},
  /* Param 6 read measured output current : AI*/
  {&DSET_AI,GPIBREAD,IB_Q_LOW,"MEAS:CURR?","%lf",0,80,0,0,0,NULL,NULL,0},
  /* Param 7 set Over Voltage Protection (OVP) value : AO*/
  {&DSET_AO,GPIBWRITE,IB_Q_HIGH,NULL,"VOLT:PROT %.3f",0,80,0,0,0,NULL,NULL,0},
  /* Param 8 read OVP setting : AI*/
  {&DSET_AI,GPIBREAD,IB_Q_LOW,"VOLT:PROT?","%lf",0,80,0,0,0,NULL,NULL,0},
  /* Param 9 Enable/Disable Over Current Protection (OCP) : BO*/
  {&DSET_BO,GPIBEFASTO,IB_Q_HIGH,"CURR:PROT:STAT ",NULL,0,80,0,0,0,offOn,&currProtStat,0},
  /* Param 10 read OCP status : BI*/
  {&DSET_BI,GPIBREAD,IB_Q_HIGH,"CURR:PROT:STAT?",0,0,80,0,0,0,NULL,&currProtStat,0},
  /* Param 11 reset Overvoltage/Overcurrent trip : BO*/
  {&DSET_BO,GPIBEFASTO,IB_Q_HIGH,"OUTP:PROT:CLE",NULL,0,80,0,0,0,NULL,&currProtStat,0},
  /* Param 12 Turn output On/Off*/
  {&DSET_BO,GPIBEFASTO,IB_Q_HIGH,"OUTPUT ",NULL,0,80,0,0,0,offOn,&outputOffOn,0},
  /* Param 13 Output On/Off : BI*/
  {&DSET_BI,GPIBREAD,IB_Q_HIGH,"OUTPUT?",0,0,80,0,0,0,NULL,&outputOffOn,0}

};

/* The following is the number of elements in the command array above.  */
#define NUMPARAMS sizeof(gpibCmds)/sizeof(struct gpibCmd)

/******************************************************************************
 * Initialize device support parameters
 *
 *****************************************************************************/
static long init_ai(int parm)
{
    if(parm==0) {
        devSupParms.name = "devAgilent6613C";
        devSupParms.gpibCmds = gpibCmds;
        devSupParms.numparams = NUMPARAMS;
        devSupParms.timeout = TIMEOUT;
        devSupParms.timeWindow = TIMEWINDOW;
        devSupParms.respond2Writes = -1;
    }
    return(0);
}
